@@ -1,4 +1,4 @@
-/* $Id: rmdraw.c,v 1.12 2004/04/07 23:05:58 rmagick Exp $ */
+/* $Id: rmdraw.c,v 1.15 2004/07/31 22:04:09 rmagick Exp $ */
 /*============================================================================\
 |                Copyright (C) 2004 by Timothy P. Hunter
 | Name:     rmdraw.c
@@ -323,6 +323,21 @@ Draw_stroke_eq(VALUE self, VALUE stroke)
     return self;
 }
 
+/*
+    Method:     Draw#stroke_width=
+    Purpose:    stroke_width attribute writer
+*/
+VALUE
+Draw_stroke_width_eq(VALUE self, VALUE stroke_width)
+{
+  Draw *draw;
+
+  rm_check_frozen(self);
+  Data_Get_Struct(self, Draw, draw);
+  draw->info->stroke_width = NUM2DBL(stroke_width);
+  return self;
+}
+
 /*
     Method:     Draw#text_antialias=
     Purpose:    text_antialias attribute writer
@@ -481,12 +496,8 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     // The default composition operator is "Over".
     if (argc == 6)
     {
-        if (TYPE(argv[5]) != T_FIXNUM)
-        {
-            rb_raise(rb_eTypeError, "composite operator must be a Fixnum (%s given)",
-                                rb_class2name(CLASS_OF(argv[5])));
-        }
         VALUE_TO_ENUM(argv[5], cop, CompositeOperator);
+
         switch(cop)
         {
             case AddCompositeOp:
@@ -556,7 +567,7 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
 
     // Create a temp copy of the composite image
     Data_Get_Struct(image, Image, comp_img);
-    write_temp_image(comp_img, name);
+    rm_write_temp_image(comp_img, name);
 
     // Add the temp filename to the filename array
     if (!draw->tmpfile_ary)
@@ -900,7 +911,7 @@ destroy_Draw(void *drawptr)
     {
         while ((tmpfile = rb_ary_shift(draw->tmpfile_ary)) != Qnil)
         {
-            delete_temp_image(STRING_PTR(tmpfile));
+            rm_delete_temp_image(STRING_PTR(tmpfile));
         }
     }
 
@@ -1153,7 +1164,7 @@ destroy_Montage(void *obj)
     // If we saved a temporary texture image, delete it now.
     if (montage->info->texture != NULL)
     {
-        delete_temp_image(montage->info->texture);
+        rm_delete_temp_image(montage->info->texture);
     }
     DestroyMontageInfo(montage->info);
     xfree(montage);
@@ -1225,7 +1236,7 @@ Montage_texture_eq(VALUE self, VALUE texture)
     Data_Get_Struct(ImageList_cur_image(texture), Image, texture_image);
 
     // Write a temp copy of the image & save its name.
-    write_temp_image(texture_image, tmpnam);
+    rm_write_temp_image(texture_image, tmpnam);
     magick_clone_string(&montage->info->texture, tmpnam);
 
     return self;
