@@ -1,4 +1,4 @@
-/* $Id: rmutil.c,v 1.32 2004/04/11 00:05:15 rmagick Exp $ */
+/* $Id: rmutil.c,v 1.41 2004/08/18 23:22:36 rmagick Exp $ */
 /*============================================================================\
 |                Copyright (C) 2004 by Timothy P. Hunter
 | Name:     rmutil.c
@@ -12,8 +12,8 @@ static const char *Compliance_Const_Name(ComplianceType *);
 static const char *StyleType_Const_Name(StyleType);
 static const char *StretchType_Const_Name(StretchType);
 static void Color_Name_to_PixelPacket(PixelPacket *, VALUE);
-
-
+static VALUE Enum_type_values(VALUE);
+static VALUE Enum_type_inspect(VALUE);
 
 /*
     Extern:     magick_malloc, magick_free, magick_realloc
@@ -68,17 +68,13 @@ void *magick_realloc(void *ptr, const size_t size)
     Extern:     magick_clone_string
     Purpose:    make a copy of a string in malloc'd memory
     Notes:      Any existing string pointed to by *new_str is freed.
-                If malloc fails, raises an exception
+                CloneString asserts if no memory. No need to check
+                clsits return value.ls
+
 */
 void magick_clone_string(char **new_str, const char *str)
 {
-    unsigned int okay;
-
-    okay = CloneString(new_str, str);
-    if (!okay)
-    {
-        rb_raise(rb_eNoMemError, "not enough memory to continue");
-    }
+    CloneString(new_str, str);
 }
 
 
@@ -133,6 +129,20 @@ char *rm_string_value_ptr_len(volatile VALUE *ptr, long *len)
 }
 
 
+/*
+ *  Extern:     rm_check_ary_len(ary, len)
+ *  Purpose:    raise exception if array too short
+*/
+void
+rm_check_ary_len(VALUE ary, int len)
+{
+    if (RARRAY(ary)->len < len)
+    {
+        rb_raise(rb_eIndexError, "not enough elements in array - expecting %d, got %d",
+                        len, RARRAY(ary)->len);
+    }
+}
+
 /*
     Extern:     rm_check_frozen
     Purpose:    backport rb_check_frozen for 1.6.x releases
@@ -157,6 +167,90 @@ rm_no_freeze(VALUE obj)
     rb_raise(rb_eTypeError, "can't freeze %s", rb_class2name(CLASS_OF(obj)));
 }
 
+
+/*
+ *  Extern:     rm_obj_to_s(obj)
+ *  Purpose:    try to convert object to string by calling its `to_s' method
+ *  Notes:      Usually run via rb_rescue so TypeError or NoMethodError can be
+ *              rescued. I use this instead of rb_str_to_str so that objects
+ *              that don't have `to_str' but do have `to_s' can be converted.
+*/
+VALUE
+rm_obj_to_s(VALUE obj)
+{
+    return TYPE(obj) == T_STRING ? obj : rb_funcall(obj, ID_to_s, 0);
+}
+
+
+
+/*
+ *  Static:     arg_is_number
+ *  Purpose:    Try to convert the argument to a double,
+ *              raise an exception if fail.
+*/
+static VALUE
+arg_is_number(VALUE arg)
+{
+    double d;
+    d = NUM2DBL(arg);
+    d = d;      // satisfy icc
+    return arg;
+}
+
+
+/*
+ *  Static:     fuzz_arg_rescue
+ *  Purpose:    called when `rb_str_to_str' raised an exception below
+*/
+static VALUE
+fuzz_arg_rescue(VALUE arg)
+{
+    rb_raise(rb_eArgError, "argument must be a number or a string in the form 'NN%' (%s given)",
+            rb_class2name(CLASS_OF(arg)));
+}
+
+
+/*
+ *  Extern:     rm_fuzz_to_dbl(obj)
+ *  Purpose:    If the argument is a number, convert it to a double.
+ *              Otherwise it's supposed to be a string in the form 'NN%'.
+ *              Return a percentage of MaxRGB.
+ *  Notes:      Called from Image#fuzz= and Info#fuzz=
+*/
+double
+rm_fuzz_to_dbl(VALUE fuzz_arg)
+{
+    double fuzz;
+    char *fuzz_str, *end;
+    int not_num;
+
+    // Try to convert the argument to a number. If failure, sets not_num to non-zero.
+    rb_protect(arg_is_number, fuzz_arg, &not_num);
+
+    if (not_num)
+    {
+        // Convert to string, issue error message if failure.
+        fuzz_arg = rb_rescue(rb_str_to_str, fuzz_arg, fuzz_arg_rescue, fuzz_arg);
+        fuzz_str = STRING_PTR(fuzz_arg);
+        fuzz = strtod(fuzz_str, &end);
+        if(*end == '%')
+        {
+            fuzz = (fuzz * MaxRGB) / 100;
+        }
+        else if(*end != '\0')
+        {
+            rb_raise(rb_eArgError, "expected percentage, got `%s'", fuzz_str);
+        }
+    }
+    else
+    {
+        fuzz = NUM2DBL(fuzz_arg);
+    }
+
+    return fuzz;
+}
+
+
 /*
     Extern:     ImageList_cur_image
     Purpose:    Sends the "cur_image" method to the object. If 'img'
@@ -826,18 +920,6 @@ Color_Name_to_PixelPacket(PixelPacket *color, VALUE name_arg)
     }
 }
 
-/*
-    Extern:     AffineMatrix_from_AffineMatrix
-    Purpose:    Create a Magick::AffineMatrix object from an AffineMatrix structure.
-*/
-VALUE
-AffineMatrix_from_AffineMatrix(AffineMatrix *am)
-{
-    return rb_funcall(Class_AffineMatrix, ID_new, 6
-                    , rb_float_new(am->sx), rb_float_new(am->rx), rb_float_new(am->ry)
-                    , rb_float_new(am->sy), rb_float_new(am->tx), rb_float_new(am->ty));
-}
-
 /*
     Extern:     AffineMatrix_to_AffineMatrix
     Purpose:    Convert a Magick::AffineMatrix object to a AffineMatrix structure.
@@ -1367,6 +1449,18 @@ Pixel_from_PixelPacket(PixelPacket *pp)
 }
 
 
+/*
+ *  Static:     color_arg_rescue
+ *  Purpose:    raise ArgumentError if the color name cannot be converted
+ *              to a string via rb_str_to_str.
+*/
+static VALUE
+color_arg_rescue(VALUE arg)
+{
+    rb_raise(rb_eTypeError, "argument must be color name or pixel (%s given)",
+            rb_class2name(CLASS_OF(arg)));
+}
+
 /*
     Extern:     Color_to_PixelPacket
     Purpose:    Convert either a String color name or
@@ -1378,21 +1472,17 @@ Color_to_PixelPacket(PixelPacket *pp, VALUE color)
     Pixel *pixel;
 
     // Allow color name or Pixel
-    if (TYPE(color) == T_STRING)
-    {
-        Color_Name_to_PixelPacket(pp, color);
-    }
-    else if (CLASS_OF(color) == Class_Pixel)
+    if (CLASS_OF(color) == Class_Pixel)
     {
         Data_Get_Struct(color, Pixel, pixel);
         *pp = *pixel;
     }
     else
     {
-        rb_raise(rb_eTypeError, "color argument must be String or Pixel (%s given)",
-                rb_class2name(CLASS_OF(color)));
+        // require 'to_str' here instead of just 'to_s'.
+        color = rb_rescue(rb_str_to_str, color, color_arg_rescue, color);
+        Color_Name_to_PixelPacket(pp, color);
     }
-
 }
 
 /*
@@ -1888,14 +1978,37 @@ TypeMetric_to_s(VALUE self)
     return rb_str_new2(buff);
 }
 
+
+/*
+ *  Extern:     rm_define_enum_type
+ *  Purpose:    set up a subclass of Enum
+*/
+VALUE rm_define_enum_type(char *tag)
+{
+    VALUE class;
+
+    class = rb_define_class_under(Module_Magick, tag, Class_Enum);\
+
+    rb_define_singleton_method(class, "values", Enum_type_values, 0);
+    rb_define_method(class, "initialize", Enum_type_initialize, 2);
+    RUBY16(rb_enable_super(class, "initialize");)
+    rb_define_method(class, "inspect", Enum_type_inspect, 0);
+    return class;
+}
+
+
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
 /*
     Extern:  rm_enum_new (1.8)
-    Purpose: Construct a new Enum instance
+    Purpose: Construct a new Enum subclass instance
 */
 VALUE rm_enum_new(VALUE class, VALUE sym, VALUE val)
 {
-   return Enum_initialize(Enum_alloc(class), sym, val);
+    VALUE argv[2];
+
+    argv[0] = sym;
+    argv[1] = val;
+    return rb_class_new_instance(2, argv, class);
 }
 
 /*
@@ -1908,10 +2021,12 @@ VALUE Enum_alloc(VALUE class)
 
    return Data_Make_Struct(class, MagickEnum, NULL, NULL, magick_enum);
 }
+
+
 #else
 /*
     Extern:  rm_enum_new (1.6)
-    Purpose: Construct a new Enum instance
+    Purpose: Construct a new Enum subclass instance
 */
 VALUE rm_enum_new(VALUE class, VALUE sym, VALUE val)
 {
@@ -1936,6 +2051,7 @@ VALUE Enum_new(VALUE class, VALUE sym, VALUE val)
     rb_obj_call_init(new_enum, 2, argv);
     return new_enum;
 }
+
 #endif
 
 /*
@@ -1953,6 +2069,7 @@ VALUE Enum_initialize(VALUE self, VALUE sym, VALUE val)
    return self;
 }
 
+
 /*
     Method:  Enum#to_s
     Purpose: Return the name of an enum
@@ -1965,6 +2082,7 @@ VALUE Enum_to_s(VALUE self)
    return rb_str_new2(rb_id2name(magick_enum->id));
 }
 
+
 /*
     Method:  Enum#to_i
     Purpose: Return the value of an enum
@@ -1977,9 +2095,10 @@ VALUE Enum_to_i(VALUE self)
    return INT2NUM(magick_enum->val);
 }
 
+
 /*
     Method:  Enum#<=>
-    Purpose: Support Enumerable module in Enum
+    Purpose: Support Comparable module in Enum
     Returns: -1, 0, 1, or nil
     Notes:   Enums must be instances of the same class to be equal.
 */
@@ -2001,7 +2120,7 @@ VALUE Enum_spaceship(VALUE self, VALUE other)
 
     // Values are equal, check class.
 
-    return rb_funcall(CLASS_OF(self), rb_intern("<=>"), 1, CLASS_OF(other));
+    return rb_funcall(CLASS_OF(self), ID_spaceship, 1, CLASS_OF(other));
 }
 
 
@@ -2026,6 +2145,69 @@ VALUE Enum_case_eq(VALUE self, VALUE other)
 }
 
 
+/*
+ *  Method:     xxx#initialize
+ *  Purpose:    initialize method for all Enum subclasses
+*/
+VALUE Enum_type_initialize(VALUE self, VALUE sym, VALUE val)
+{
+    volatile VALUE super_argv[2];
+    volatile VALUE enumerators;
+
+    super_argv[0] = sym;
+    super_argv[1] = val;
+    rb_call_super(2, (VALUE *)super_argv);
+
+    if (rb_cvar_defined(CLASS_OF(self), ID_enumerators) != Qtrue)
+    {
+        RUBY18(rb_cvar_set(CLASS_OF(self), ID_enumerators, rb_ary_new(), 0));
+        RUBY16(rb_cvar_set(CLASS_OF(self), ID_enumerators, rb_ary_new()));
+    }
+
+    enumerators = rb_cvar_get(CLASS_OF(self), ID_enumerators);
+    rb_ary_push(enumerators, self);
+
+    return self;
+}
+
+
+/*
+ *  Method:     xxx#inspect
+ *  Purpose:    Enum subclass #inspect
+*/
+static VALUE Enum_type_inspect(VALUE self)
+{
+    char str[100];
+    MagickEnum *magick_enum;
+
+    Data_Get_Struct(self, MagickEnum, magick_enum);
+    sprintf(str, "%.16s=%d", rb_id2name(magick_enum->id), magick_enum->val);
+
+    return rb_str_new2(str);
+}
+
+
+/*
+ *  Method:     xxx.each
+ *  Purpose:    singleton iterator over enumerators list
+ *  Notes:      defined for each Enum subclass
+*/
+static VALUE Enum_type_values(VALUE class)
+{
+    volatile VALUE enumerators;
+    int x;
+
+    enumerators = rb_cvar_get(class, ID_enumerators);
+
+    for (x = 0; x < RARRAY(enumerators)->len; x++)
+    {
+        rb_yield(rb_ary_entry(enumerators, x));
+    }
+
+    return class;
+}
+
+
 /*
     Static:     Compliance_Const_Name
     Purpose:    Return the string representation of a ComplianceType value
@@ -2192,7 +2374,7 @@ StyleType_Const_Name(StyleType style)
                 of size MaxTextExtent.
 */
 void
-write_temp_image(Image *image, char *tmpnam)
+rm_write_temp_image(Image *image, char *tmpnam)
 {
     long registry_id;
 
@@ -2213,7 +2395,7 @@ write_temp_image(Image *image, char *tmpnam)
 */
 
 void
-delete_temp_image(char *tmpnam)
+rm_delete_temp_image(char *tmpnam)
 {
     long registry_id = -1;
 
@@ -2225,7 +2407,7 @@ delete_temp_image(char *tmpnam)
 }
 
 /*
-    External:   not_implemented
+    External:   rm_not_implemented
     Purpose:    raise NotImplementedError
     Notes:      Called when a xMagick API is not available.
                 Replaces Ruby's rb_notimplement function.
@@ -2233,12 +2415,12 @@ delete_temp_image(char *tmpnam)
                 until 5.5.7. Use MAGICKNAME instead.
 */
 void
-not_implemented(const char *method)
+rm_not_implemented(void)
 {
 
-    rb_raise(rb_eNotImpError, "the %s method is not supported by "
+    rb_raise(rb_eNotImpError, "the `%s' method is not supported by "
                               Q(MAGICKNAME) " " MagickLibVersionText
-                              , method);
+                              , rb_id2name(rb_frame_last_func()));
 }
 
 /*
@@ -2300,7 +2482,7 @@ ImageMagickError_initialize(int argc, VALUE *argv, VALUE self)
  *              them in C variables.
 */
 void
-get_geometry(
+rm_get_geometry(
     VALUE geom,
     long *x,
     long *y,
@@ -2420,14 +2602,15 @@ magick_error_handler(
                 If the exception is an error, DOES NOT RETURN!
 */
 void
-handle_error(ExceptionInfo *ex)
+rm_handle_error(ExceptionInfo *ex)
 {
+#define RM_MAX_ERROR_CLAUSE 250
     ExceptionType sev = ex->severity;
-    char reason[251];
-    char desc[251];
+    char reason[RM_MAX_ERROR_CLAUSE+1];
+    char desc[RM_MAX_ERROR_CLAUSE+1];
 
 #if defined(HAVE_EXCEPTIONINFO_MODULE)
-    char module[251], function[251];
+    char module[RM_MAX_ERROR_CLAUSE+1], function[RM_MAX_ERROR_CLAUSE+1];
     unsigned long line;
 #endif
 
@@ -2440,12 +2623,12 @@ handle_error(ExceptionInfo *ex)
     }
     if (ex->reason)
     {
-        strncpy(reason, ex->reason, 250);
+        strncpy(reason, ex->reason, RM_MAX_ERROR_CLAUSE);
         reason[250] = '\0';
     }
     if (ex->description)
     {
-        strncpy(desc, ex->description, 250);
+        strncpy(desc, ex->description, RM_MAX_ERROR_CLAUSE);
         desc[250] = '\0';
     }
 
@@ -2455,12 +2638,12 @@ handle_error(ExceptionInfo *ex)
 
     if (ex->module)
     {
-        strncpy(module, ex->module, 250);
+        strncpy(module, ex->module, RM_MAX_ERROR_CLAUSE);
         module[250] = '\0';
     }
     if (ex->function)
     {
-        strncpy(function, ex->function, 250);
+        strncpy(function, ex->function, RM_MAX_ERROR_CLAUSE);
         function[250] = '\0';
     }
     line = ex->line;
@@ -2486,7 +2669,7 @@ handle_error(ExceptionInfo *ex)
                 image has an error, raise an exception. Otherwise
                 if any image has a warning, issue a warning message.
 */
-void handle_all_errors(Image *seq)
+void rm_handle_all_errors(Image *seq)
 {
     Image *badboy = NULL;
     Image *image = seq;
@@ -2513,9 +2696,9 @@ void handle_all_errors(Image *seq)
     {
         if (badboy->exception.severity > WarningException)
         {
-            unseq(seq);
+            rm_unseq(seq);
         }
-        handle_error(&badboy->exception);
+        rm_handle_error(&badboy->exception);
     }
 }
 
@@ -2528,7 +2711,7 @@ void handle_all_errors(Image *seq)
     Returns:    a pointer to the head of the scene sequence list
 */
 Image *
-toseq(VALUE imagelist)
+rm_toseq(VALUE imagelist)
 {
     long x, len;
     Image *head = NULL;
@@ -2574,7 +2757,7 @@ toseq(VALUE imagelist)
     Notes:      The images remain grouped via the ImageList
 */
 void
-unseq(Image *image)
+rm_unseq(Image *image)
 {
 
     if (!image)
